'use strict';

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function _objectSpread(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};
    var ownKeys = Object.keys(source);

    if (typeof Object.getOwnPropertySymbols === 'function') {
      ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) {
        return Object.getOwnPropertyDescriptor(source, sym).enumerable;
      }));
    }

    ownKeys.forEach(function (key) {
      _defineProperty(target, key, source[key]);
    });
  }

  return target;
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

function _objectWithoutProperties(source, excluded) {
  if (source == null) return {};

  var target = _objectWithoutPropertiesLoose(source, excluded);

  var key, i;

  if (Object.getOwnPropertySymbols) {
    var sourceSymbolKeys = Object.getOwnPropertySymbols(source);

    for (i = 0; i < sourceSymbolKeys.length; i++) {
      key = sourceSymbolKeys[i];
      if (excluded.indexOf(key) >= 0) continue;
      if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue;
      target[key] = source[key];
    }
  }

  return target;
}

function _slicedToArray(arr, i) {
  return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest();
}

function _arrayWithHoles(arr) {
  if (Array.isArray(arr)) return arr;
}

function _iterableToArrayLimit(arr, i) {
  var _arr = [];
  var _n = true;
  var _d = false;
  var _e = undefined;

  try {
    for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

function _nonIterableRest() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance");
}

var ActionStatus = {
  __THUNKLESS_ACTION_STATUS_BUSY__: Symbol('__THUNKLESS_ACTION_STATUS_BUSY__'),
  __THUNKLESS_ACTION_STATUS_SUCCESS__: Symbol('__THUNKLESS_ACTION_STATUS_SUCCESS__'),
  __THUNKLESS_ACTION_STATUS_FAILURE__: Symbol('__THUNKLESS_ACTION_STATUS_FAILURE__')
};
var values = new Set(Object.values(ActionStatus));
var getDescription = Object.getOwnPropertyDescriptor(Symbol.prototype, 'description') ? function (symbol) {
  return symbol.description;
} : function (symbol) {
  return symbol.toString().slice(7, -1);
};
var ActionStatus$1 = {
  BUSY: ActionStatus.__THUNKLESS_ACTION_STATUS_BUSY__,
  SUCCESS: ActionStatus.__THUNKLESS_ACTION_STATUS_SUCCESS__,
  FAILURE: ActionStatus.__THUNKLESS_ACTION_STATUS_FAILURE__,
  toString: getDescription,
  fromString: function fromString(value) {
    return ActionStatus[value];
  },
  has: Set.prototype.has.bind(values)
};

var prepareChain = function prepareChain(chain, payload) {
  return Promise.all([payload, typeof chain === 'function' ? chain(payload) : chain]);
};

var createDispatcher = function createDispatcher(store, next, extra, type) {
  return function (_ref) {
    var _ref2 = _slicedToArray(_ref, 2),
        payload = _ref2[0],
        chain = _ref2[1];

    if (chain) {
      if (chain instanceof Array) {
        for (var i = 0; i < chain.length; i++) {
          if (chain[i]) store.dispatch(chain[i]);
        }
      } else {
        store.dispatch(chain);
      }
    }

    return next(_objectSpread({}, extra, {
      payload: payload,
      type: type
    }));
  };
};

var createErrorHandler = function createErrorHandler(store, next, failureType, dispatchOnError, action, extra) {
  return function (error) {
    var payload = {
      error: error,
      origin: action
    };
    var nextval = next(_objectSpread({}, extra, {
      type: failureType,
      error: true,
      payload: payload
    }));

    if (dispatchOnError) {
      if (typeof dispatchOnError === 'string') {
        store.dispatch({
          type: dispatchOnError,
          error: true,
          payload: payload
        });
      } else {
        var errorAction = dispatchOnError(payload);
        if (errorAction) store.dispatch(errorAction);
      }
    }

    return nextval;
  };
};

var createPromiseResolver = (function (store) {
  return function (next, promise, action, successType, failureType, chain, dispatchOnError, extra) {
    return promise.then(function (payload) {
      return prepareChain(chain, payload);
    }).then(createDispatcher(store, next, extra, successType)).catch(createErrorHandler(store, next, failureType, dispatchOnError, action, extra));
  };
});

var HOP = Object.prototype.hasOwnProperty;

var createMiddleware = function createMiddleware(store, next, resolve) {
  return function (action) {
    if (!HOP.call(action, 'promise')) {
      if (typeof action.transform !== 'function') return next(action);
      return next(action.transform(action, store.getState()));
    }

    var type = action.type,
        error = action.error,
        payload = action.payload,
        promise = action.promise,
        statusSelector = action.statusSelector,
        chain = action.chain,
        dispatchOnError = action.dispatchOnError,
        transform = action.transform,
        extra = _objectWithoutProperties(action, ["type", "error", "payload", "promise", "statusSelector", "chain", "dispatchOnError", "transform"]);

    if (action.error === true) return next(action);

    if (statusSelector) {
      // Blocking  action
      if (!(type instanceof Array)) {
        throw new Error('`type` must be Array if `statusSelector` is defined');
      }

      var state = store.getState();

      if (!(statusSelector instanceof Array)) {
        // Single status selector
        var currentStatus = statusSelector(state);

        if (currentStatus === ActionStatus$1.BUSY || currentStatus === false) {
          return null;
        }
      } else {
        // Block action if any status selector returns BUSY
        for (var i = 0; i < statusSelector.length; i++) {
          var _currentStatus = statusSelector[i](state);

          if (_currentStatus === ActionStatus$1.BUSY || _currentStatus === false) {
            return null;
          }
        }
      }
    } // If action.transform function is set, apply it before passing action to next.


    var _next = typeof transform === 'function' ? function (action) {
      return next(transform(action, store.getState()));
    } : next;

    var successType;
    var failureType;

    if (type instanceof Array) {
      _next(_objectSpread({
        type: type[0],
        payload: payload
      }, extra)); // Dispatch start action


      successType = type[1];
      failureType = type[2];
    } else {
      successType = failureType = type;
    } // At this point, start action has been reduced and status has been updated if necessary


    var _promise = typeof promise === 'function' ? promise() : promise;

    return resolve(_next, _promise, action, successType, failureType, chain, dispatchOnError, extra);
  };
};

var thunkless = (function (store) {
  return function (next) {
    return createMiddleware(store, next, createPromiseResolver(store));
  };
});

var index = Object.assign(thunkless, {
  ActionStatus: ActionStatus$1
});

module.exports = index;
